import os

if __name__ == "__main__":
    from price_val_engine import cmd
    cmd.run_from_commandline()
    
    
    # engine = engine.Engine(
    #     input_file_path="input.csv",
    #     output_file_path="output.csv"
    # )
    # engine.validate_all()
    # engine.save()

